function subs = forwardSelection(df, maxSize)
%function subs = subsetSelection(df, maxSize, type)
%
%Do subset selection on df, using the forward selection heuristic, looking
%at subsets of increasing size. 
%Returns a struct array of size 1..maxSize, containing selected features
%vector (colidx), rmse and mae of the model (on training data)

error(nargchk(2,2,nargin));

display (sprintf('Forward selection started: %d features total',size(df.traindata.features,2)));
X = ones(size(df.traindata.features,1),1);
global delta; % = 1e-16;
A = X'*X+delta;
Ainv = inv(A);
XtY = X' * df.traindata.response;
compute_training_RMSE = false;

%test = feval(df.backtransform, df.validdata.response, df);
test = df.validdata.response;

features = [];
num_train_instances = size(df.traindata.features,1);
num_test_instances = size(df.validdata.features,1);
num_features = size(df.traindata.features,2);
maxIteration = min(maxSize,num_features);
Xtest = [ones(num_test_instances,1) zeros(num_test_instances,maxIteration+1)];

% for every set size
for t=1:maxIteration
	
	minRMSE = inf;
	best_subset = 0;

	if (mod(t,5)==0)
		display (sprintf('  forward selection: subset size %d of %d', [t maxIteration]));
	end
	
	% for each feature not in the set
	for tt=1:size(df.traindata.features,2) %- size(features,2)
		if (~isempty(find(features==tt)))
			continue;
		end
		
		features(t)=tt;
	
		% construct the model with this feature added
		model = modelAddFeature(X, df.traindata.features(:,tt), df.traindata.response, A, Ainv, delta, XtY);

		% record the RMSE and MAE on validation data for this feature
		Xtest(:,t+1) = df.validdata.features(:,tt);
		preds = Xtest*[model; zeros(maxIteration - t+1,1)];
		
		% are we sure we want to do this?  This means we're doing model
		% selection based on a different error measure from the one we
		% optimize in the regression...
%		preds = feval(df.backtransform, preds, df);

		result(tt).MAE = mean(abs(preds-test));
		result(tt).RMSE = sqrt(mean( (preds-test).^2 ));

		if (compute_training_RMSE)
			preds = [X df.traindata.features(:,tt)]*model;
			%preds = feval(df.backtransform, preds, df);
			%result(tt).trainingRMSE = sqrt(mean((preds-feval(df.backtransform, df.traindata.response, df)).^2));
			result(tt).trainingRMSE = sqrt(mean((preds-df.traindata.response).^2));
		end
		
		result(tt).features = features;
		result(tt).method = 'forwardSelection';
		minRMSE = min(minRMSE, result(tt).RMSE);
		if (minRMSE == result(tt).RMSE)
			best_subset = tt;
		end
	end

	% pick the feature with the best RMSE
	subs(t) = result(best_subset);
	clear result;
	features = subs(t).features;
	
	% update the inverse; global variables
	[model A Ainv XtY] = modelAddFeature(X, df.traindata.features(:,best_subset), df.traindata.response, A, Ainv, delta, XtY);
	X = [X df.traindata.features(:,best_subset)];
	Xtest(:,t+1) = df.validdata.features(:,best_subset);
end

display 'Forward selection finished';