%pipline_exec.m

%

%The actual executable part of the pipeline, with params set elsewhere.

%Useful for batch jobs.





% ========================================================================

% ========================================================================

% These are still editable, but probably shouldn't be touched...


outputdir=[location 'output/'];

warning off MATLAB:nearlySingularMatrix

global outputPrefix;

outputPrefix = [outputdir currDataFile];

linearPreprocessFile = [outputPrefix '-preprocessed.mat'];

quadPreprocessFile = [outputPrefix '-quad-preprocessed.mat'];

if doQuadratic outputPrefix=[outputPrefix '-quad']; end;

subsetOutFile = [outputPrefix '-subsets'];

vimpOutFile = [outputPrefix '-vimp'];

statsOutFile = [outputPrefix '-stats-'];

portOutFile = [outputPrefix '-portstats.txt'];

trainfilename = [location currDataFile '-train.csv'];

validfilename = [location currDataFile '-valid.csv'];

if useValidSet 
	testfilename = [location currDataFile '-valid.csv'];
else 
	testfilename = [location currDataFile '-test.csv'];
end



if nAlgs > 1
    doPortfolio = true;
    cappedIncluded = true;
end



% The following variables are defined as global, as are some above.

% What this means is that any called function which *also declares them as global* will have access to these variables.  If called functions have

% variables of the same name which are not declared as global, they are different local variables.

global df;

global subs;

original_delta = delta;

%if (doVariableImportance == false) doSubplots = false; end

% =======================================================================
% Execution of the pipeline starts here
% =======================================================================

%read data
alldf=readDF(trainfilename, testfilename, validfilename, nAlgs, 0.005, cappedIncluded, doPortfolio, true); % 0.005 is the mintime - smaller runtimes are rounded up

tic   %process timing
clear stats; %for batch processing
for alg=1:nAlgs
    delta = original_delta;
    % transform the response variable

    df = transformData(alldf(alg), modelTransformations{transformation});

    display(sprintf('%s tranformation applied to response variable',modelTransformations{transformation}));

	% =====================================================================
	% Linear features
	% =======================================================================

	% throw away features that are strongly linearly correlated
% 	display (sprintf('Linear features: Throwing away features with linear
% 	correlations above %f',LinearCorrelationCutoff));
% 	df = deleteLinearCorrelations(df,LinearCorrelationCutoff);

 	% throw away constant features
% 	df = deleteConstantFeatures(df,1e-8);
	% throw away features one at a time if matrix is ill-conditioned
	% caution: throwing away too much hurts performance
	% especially, don't throw away too many linear features
% 	display 'Checking to see if the matrix is ill-conditioned'
% 	df = deleteIllConditionedFeatures(df,'linear',LinearConditionNumberCutoff);

    if doLinearPreprocessing
        [df ppLinRes]=preprocessFeatures(df, linearR2Cutoff, linearPreprocessFile);
    end

	%Select small number of features on validation set
	if (linSize < size(df.testdata.features,2))
		display (sprintf('Performing subset selection to select %d of %d linear features...',linSize,size(df.testdata.features,2)));
		subs = subsetSelection(df, linSize, subsetSelectionMethods{linearSelectionMethod}, true);
		df = projectDF(df, subs(linSize).features);
	end

	% =====================================================================
	% Quadratic features
	% =======================================================================

	if (doQuadratic)
		%square it
		display 'Generating quadratic features...'
		df = squareDF(df);

		%Save raw quadratic results just in case!
		%saveDF(['quad-' df.algname '-' trainfilename],  ['quad-' df.algname '-' testfilename], df,  ['quad-' df.algname '-' validfilename]);

		% throw away features that are strongly linearly correlated
%		display (sprintf('Quadratic features: Throwing away features with
%		linear correlations above %f',QuadraticCorrelationCutoff));
%		deleteLinearCorrelations(df,QuadraticCorrelationCutoff);

		% throw away features one at a time if matrix is ill-conditioned
		% caution: throwing away too much hurts performance
% 		display 'Checking to see if the matrix is ill-conditioned'
% 		deleteIllConditionedFeatures(df,'quadratic',QuadraticConditionNumberCutoff);

		% continue to double delta if matrix is ill-conditioned
% 		increaseDeltaIfIllConditioned(df, QuadraticConditionNumberCutoffDelta);

       if doQuadPreprocessing
           [df ppQuadRes]=preprocessFeatures(df, quadR2Cutoff, quadPreprocessFile);
       end

		%reduce it even further on valid set
		if (quadSize < size(df.traindata.features,2))
			display (sprintf('Performing subset selection to select %d of %d quadratic features...', quadSize, size(df.traindata.features,2)));
			subs = subsetSelection(df, quadSize, subsetSelectionMethods{quadraticSelectionMethod}, true);
			df = projectDF(df, subs(quadSize).features);
        end
	end

	% =====================================================================
	% Variable importance
	% =======================================================================

    if (doVariableImportance)
		display 'Variable importance: evaluating subsets of the data constructed above...'
		subs = subsetSelection(df, variableSelectionMaxSubsetSize, subsetSelectionMethods{variableImportanceSelectionMethod}, true);
		%Output subsets into a file
		writeSubsetFile(subsetOutFile, subs, df);

        %Select size and do var importance
		if (variableImportanceSubsetSize == 0)
			rmses = [subs.RMSE];

			display 'You can type varimp(x) to view the bar graph of the importance of the variables in the best subset of size x';
            display 'Results can be saved with writeVimpFile(vimpOutFile, vimp, df)';

%  			ind=max(4,min(find( ((rmses - min(rmses)) / min(rmses)) <= vImpThreshold )));

            diff = rmses(2:end) - rmses(1:end-1);
            for tt = 1:size(diff,2)
                ratio(tt) = diff(tt)/mean(diff(tt:end));
            end
            ind=min(10,max(4,min(find(ratio <= vImpThreshold))));
            if (size(find(ratio <= vImpThreshold),2))==0
                ind=10;
            end
        else
			ind = variableImportanceSubsetSize;
		end

		
		vimp = varImportance(df, subs, ind, true, true, doSubplots, doBackTransform);

		%output var importance info into a file

		writeVimpFile(vimpOutFile, vimp, df);		

		% necessary features

		temp = [vimp.featurenum];

		nec = necessaryFeatures(df, subs(ind), temp(end), necessary_feature_threshold);

        diary([statsOutFile df.algname '.txt']);

		display (sprintf('The validation-set RMSE of this %d-variable model was %f.',ind,subs(ind).RMSE));

		display (sprintf('The most important feature was %s.  ',df.featurenames{temp(end)}));

		if (size(nec,2)==1)

			display (sprintf('No other features can substitute for %s in the same %d variable model, keeping RMSE within %d%%.',df.featurenames{temp(end)},ind,necessary_feature_threshold*100))

		else

			display (sprintf('Other features that can substitute for %s in the same %d variable model, keeping RMSE within %d%%:',df.featurenames{temp(end)},ind,necessary_feature_threshold*100))

			for i=2:size(nec,2)

				display(sprintf('  %s --- %f RMSE',df.featurenames{nec(i).feature},nec(i).RMSE))

			end

		end

		display(sprintf('\n'))

        diary off;

	end

	

	% =======================================================================

	% Report regression results

	% =======================================================================



	%report results

	stats(alg) = modelStats(df, false, doBackTransform);

    diary([statsOutFile df.algname '.txt']);

	disp(['Reporting Statistics for ' df.algname]);

	disp(stats(alg));
    if doVariableImportance
    fprintf('variable importance report:\n');
     fprintf('------------------------------------\n');
    for i=1:ind
        fprintf('%s: %f\n', vimp(ind+1-i).feature, vimp(ind+1-i).importance );
    end
    fprintf('------------------------------------\n');
    end
    %randomization test

    if ~doPortfolio
        if doRandomizationTest
            nullhyp = stats(alg).testmean * ones(size(stats(alg).preds,1),1);

            [p errorDist]=randomizationTest(nullhyp, stats(alg).preds, randomAmt);

            disp('NOT SURE IF THE FOLLOWING STATEMENT IS MEANINGFULL!!!!!');

            disp(sprintf('Prediction better than mean by %f with probability %f ', randomAmt, p));

        end

        if saveFigs && doRandomizationTest

            saveMatrix(errorDist, {'Error Distribution'}, [outputPrefix '-rt-err-dist-' df.algname '.csv']);

        end

        

    end

    diary off;

    



	% display figures

    if ~doPortfolio
       if(doSubplots) subplot(2,2,4);
        else figure; 
       end

        if (doBackTransform)

            loglog(max(stats(alg).preds,1e-5), feval(df.backtransform, df.testdata.response, df),'.')

        else

            if (strcmp(modelTransformations{transformation},'identity'))

                loglog( df.testdata.response, max(stats(alg).preds,1e-5),'.')

            else

                plot( df.testdata.response,max(stats(alg).preds,-5),'.')

            end

        end

        xlabel ('Actual Log Runtime');

        ylabel ('Predicted Log Runtime');

        title ('Scatterplot for full model');

        

        if saveFigs

            if doSubplots name=[outputPrefix '-subplots-' df.algname]; else name=[outputPrefix '-scatter-' df.algname]; end

            saveCurrFigure(name);

        end

        

        %Histogram of error - for now, always without subplot

        figure;
        linfoo= (df.testdata.response - stats(alg).preds).^2;
        linfoo=sort(linfoo);
        linyvalue=[1:size(linfoo,1)]/size(linfoo,1);
        plot(linfoo, linyvalue);
        
        %hist((df.testdata.response - stats(alg).preds).^2,20);

        xlabel('Squared Error');

        ylabel('Fraction Below');

        title ('Squared Error CDF');

        if saveFigs

            saveCurrFigure([outputPrefix '-error-cdf-' df.algname]);

        end

        

        %Save data for figures

        if saveFigs

            if doBackTransform

                actual = feval(df.backtransform, df.testdata.response, df);

            else

                actual = df.testdata.response;

            end

            saveMatrix([actual stats(alg).preds], {'Actual' 'Predictions'}, [outputPrefix '-pred-data-' df.algname '.csv']);

        end

    end

    

end



% =======================================================================

% Portfolio computations

% =======================================================================

if doPortfolio
    portstats = portfolioStats(stats, nAlgs, {alldf.algname});

    diary(portOutFile);

    disp('Reporting Portfolio Statistics');

    disp(portstats);

    diary off;

end



toc

