function portstats=portfolioStats(stats, numalgs, algnames)
%function portstats=portfolioStats(stats, numalgs, algnames)
%
%Compute various statistics pertaining to the portfolio
global doSubplots;
global saveFigs;
global outputPrefix;

error(nargchk(3,3,nargin));

portstats.algnames = algnames;

%average runtimes
runtimes=[stats.runtimes];
portstats.averages = mean(runtimes);

%portfolio runtimes
[opttime, optpick] = min(runtimes,[],2);
[dummy, portpick] = min([stats.preds],[],2);

portstats.optPortfolio = mean(opttime);

n=size(portpick,1);


for t=1:n
    porttime(t,1) = runtimes(t,portpick(t));
end

portstats.portRuntime = mean(porttime);
portstats.portTime = mean(porttime + stats(1).featureTimes);
portstats.featTime = mean(stats(1).featureTimes);

%pick percentages
for t =1:numalgs
    portstats.optFraction(t) = size(find(optpick==t),1) / n;
    portstats.portFraction(t) = size(find(portpick==t),1) / n;
end

%Other metrics
portstats.percentOptimal = mean(optpick==portpick);
portstats.avgPercSubopt = mean((porttime - opttime)./opttime);

%Now draw figures

%First, averages
figure;
if (doSubplots) subplot(2,2,1); end;

bar(portstats.averages);
set(gca, 'XTickLabel', algnames);
ylabel('Runtime');
title('Average Runtimes');
if (saveFigs && ~doSubplots) saveCurrFigure([outputPrefix '-avg-runtimes']); end

%Then portfolios
if doSubplots subplot(2,2,2); else figure; end;

[bestAlg, bestIdx] = min(portstats.averages);
data(1,:)=[bestAlg, portstats.optPortfolio, portstats.portRuntime];
data(2,:)=[0,0,portstats.featTime];
bar(data', 'stacked');
set(gca, 'XTickLabel', {algnames{bestIdx} 'Optimal' 'Portfolio'});
ylabel('Runtime');
title('Portfolio Runtimes');
if (saveFigs && ~doSubplots) saveCurrFigure([outputPrefix '-port-runtimes']); end

%Pie charts
if doSubplots subplot(2,2,3); else figure; end;
pie(portstats.optFraction, algnames);
title('Optimal Pick');
if (saveFigs && ~doSubplots) saveCurrFigure([outputPrefix '-opt-pie']); end

if doSubplots subplot(2,2,4); else figure; end;
pie(portstats.portFraction, algnames);
title('Actual Pick');
if (saveFigs && ~doSubplots) saveCurrFigure([outputPrefix '-port-pie']); end

if(saveFigs && doSubplots) saveCurrFigure([outputPrefix '-portfolio-fig']); end;