function vimp=varImportance(df, subs, ind, useValidSet, outputFirstFigure, doSubplots, doBackTransform)
%function vimp=varImportance(df)
%
%Do variable importance, by collecting stats for each leave-one variable
%model.

global modelTransformations;
global transformation;
global saveFigs;
global outputPrefix;

error(nargchk(7,7,nargin));

df=projectDF(df, subs(ind).features);

n=size(df.traindata.features,2);

fullStats = modelStats(df, useValidSet, false);

for i=1:n
    colidx=((1:n)~=i);
    tmp = projectDF(df, colidx);
    stats = modelStats(tmp, useValidSet, false);
    imp(i) = stats.trainRMSE - fullStats.trainRMSE;
end

%normalize
imp = 100 * (imp ./ max(imp));

%get indices in ascending order
[tmp idx] = sort(imp);

% keep track of the original feature numbers that correspond to these indexes
f = subs(ind).features;
featurenum = zeros(1,n);
for i=1:n
	featurenum(i) = f(idx(i));
end

% build the variable importance structure
vimp = struct('feature', df.featurenames(idx), 'featurenum',num2cell(featurenum),'importance', num2cell(imp(idx)));

% subset size figure
if (outputFirstFigure)
	figure;
	if (doSubplots) subplot(2,2,1); end
	plot(1:size([subs.RMSE],2),[subs.RMSE],'k.-');
	xlabel('subset size');
	ylabel('Validation Set RMSE');
	title 'Validation RMSE on Subsets of Different Sizes';
	axis;  % store the axis in "ans" - x is ans(1:2), y is ans(3:4)
	temp1=ans;
	temp = [subs.RMSE];
	AChibi = arrow([ind temp(ind)+.02*(temp1(4)-temp1(3))],[ind+.4 temp(ind)+.1*(temp1(4)-temp1(3))],'ends','start');
	if (doSubplots)
		arrow(AChibi,'FaceColor','r','EdgeColor','k','tipangle',20,'length',5,'width',1)
		text(ind+.4, temp(ind)+.12*(temp1(4)-temp1(3)),'Selected subset','Color','k','FontName','utopia','FontSize',7')
	else
		arrow(AChibi,'FaceColor','r','EdgeColor','k','tipangle',30,'length',20,'width',5)
		text(ind+.4, temp(ind)+.12*(temp1(4)-temp1(3)),'Selected subset','Color','k','FontName','utopia','FontSize',10')
	end
    
    if (saveFigs && ~doSubplots) saveCurrFigure([outputPrefix '-subsets-' df.algname]); end
end

% variable importance figure
if(doSubplots) subplot(2,2,2);
else figure; end
barh([vimp.importance]);
if(~doSubplots) 
	set(gca,'YTickLabel',char({vimp.feature}')); 
else
	most_important_features_in_descending_order = char({vimp.feature}');
	most_important_features_in_descending_order = most_important_features_in_descending_order(end:-1:1,:)  % they actually start out in *ascending* order!
end
title (sprintf('Relative Importance of Variables in Subset of Size %d (validation data)',size([vimp.importance],2)));
xlabel 'Relative Importance';
if (saveFigs && ~doSubplots) saveCurrFigure([outputPrefix '-vimp-' df.algname]); end

% scatterplot 
if(doSubplots) subplot(2,2,3);
else figure; end

if useValidSet
    response = df.validdata.response;
else
    response = df.testdata.response;
end

if (doBackTransform)
	loglog(max(stats.preds,1e-5), feval(df.backtransform, response, df),'.')
else
	if (strcmp(modelTransformations{transformation},'identity'))
		loglog(max(stats.preds,1e-5), response,'.')
	else
		plot(max(stats.preds,-5), response,'.')
	end
end
xlabel ('Predictions');
ylabel ('Actual');
title ('Scatterplot for subset model');

if (saveFigs && ~doSubplots) saveCurrFigure([outputPrefix '-subset-scatter-' df.algname]); end